@@ -1,5 +1,5 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2012.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2014.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
 */
 
@@ -131,11 +131,20 @@ int OTF_CopyHandler_DefSclFile(  void* userData, uint32_t stream,
 }
 
 
+int OTF_CopyHandler_DefUniqueId( void* userData, uint32_t stream,
+		uint64_t uid ) {
+
+	/* unique-id record is written implicitly */
+
+	return OTF_RETURN_OK; /* success */
+}
+
+
 int OTF_CopyHandler_DefVersion( void* userData, uint32_t stream, 
 		uint8_t major, uint8_t minor, uint8_t sub, const char* string ) {
 
 
-	/* version is writen implicitly */
+	/* version record is written implicitly */
 
 	return OTF_RETURN_OK; /* success */
 }
@@ -203,6 +212,20 @@ int OTF_CopyHandler_DefProcessSubstitutes( void* userData, uint32_t stream,
 		list ) ) ? OTF_RETURN_ABORT : OTF_RETURN_OK;
 }
 
+int OTF_CopyHandler_DefAuxSamplePoint( void*                  userData,
+                                       uint32_t               stream,
+                                       uint64_t               time,
+                                       OTF_AuxSamplePointType type,
+                                       OTF_KeyValueList*      list ) {
+
+
+    return ( 0 == OTF_Writer_writeDefAuxSamplePoint( (OTF_Writer*)userData,
+            stream,
+            time,
+            type,
+            list ) ) ? OTF_RETURN_ABORT : OTF_RETURN_OK;
+}
+
 /* *** Event handlers *** ****************************************** */
 
 int OTF_CopyHandler_NoOp( void* userData, uint64_t time, uint32_t process,
@@ -470,6 +493,40 @@ int OTF_CopyHandler_BeginFileOpSnapshot( void* userData, uint64_t time,
 		 ) ? OTF_RETURN_ABORT : OTF_RETURN_OK;
 
 }
+
+
+int OTF_CopyHandler_CollopCountSnapshot( void* userData,
+                                         uint64_t time,
+                                         uint32_t process,
+                                         uint32_t communicator,
+                                         uint64_t count,
+                                         OTF_KeyValueList* list ) {
+
+    return ( 0 == OTF_Writer_writeCollopCountSnapshot( (OTF_Writer*)userData,
+        time,
+        process,
+        communicator,
+        count,
+        list ) ) ? OTF_RETURN_ABORT : OTF_RETURN_OK;
+}
+
+int OTF_CopyHandler_CounterSnapshot( void*             userData,
+                                     uint64_t          time,
+                                     uint64_t          originaltime,
+                                     uint32_t          process,
+                                     uint32_t          counter,
+                                     uint64_t          value,
+                                     OTF_KeyValueList* list ) {
+
+    return ( 0 == OTF_Writer_writeCounterSnapshot( (OTF_Writer*)userData,
+        time,
+        originaltime,
+        process,
+        counter,
+        value,
+        list ) ) ? OTF_RETURN_ABORT : OTF_RETURN_OK;
+}
+
 /* *** summary handlers ********************************************** */
 int OTF_CopyHandler_SummaryComment( void * userData, uint64_t time,
 	uint32_t process, const char* comment, OTF_KeyValueList* list ) {
@@ -561,7 +618,7 @@ int OTF_CopyHandler_DefMarker( void *userData, uint32_t stream,
 	stream= 0;
 
 	return ( 0 == OTF_Writer_writeDefMarkerKV( (OTF_Writer*) userData,
-		stream, token, name, type, list ) ) ? OTF_RETURN_ABORT : OTF_RETURN_OK;;
+		stream, token, name, type, list ) ) ? OTF_RETURN_ABORT : OTF_RETURN_OK;
 }
 
 
