@@ -1,8 +1,9 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2012.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2014.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch, Johannes Spazier
 */
 
+#include "OTF_Platform.h"
 #include "OTF_KeyValue.h"
 #include "OTF_Errno.h"
 
@@ -85,6 +86,17 @@ uint8_t OTF_KeyValueList_close(OTF_KeyValueList* list) {
 	return 0;
 }
 
+OTF_KeyValueList* OTF_KeyValueList_clone(OTF_KeyValueList* list) {
+
+	OTF_KeyValueList *new_list;
+	new_list = OTF_KeyValueList_new();
+	if ( new_list == NULL || list == NULL ) {
+		return new_list;
+	}
+	OTF_KeyValueList_appendKeyValueList( new_list, list );
+	return new_list;
+}
+
 uint8_t OTF_KeyValueList_reset(OTF_KeyValueList* list) {
 	
 	/*OTF_KeyValuePairList *next;
@@ -219,7 +231,7 @@ uint8_t OTF_KeyValueList_appendPair(OTF_KeyValueList* list, OTF_KeyValuePair pai
 
 	p->kvPair = pair;
 	list->kvCurrent = p->kvNext;
-    
+
 	list->count++;
 
 	return 0;
@@ -808,7 +820,13 @@ uint8_t OTF_KeyValueList_removeKey(OTF_KeyValueList *list, uint32_t key) {
 			if ( p->kvNext ) {
 				p->kvNext->kvPrev = p->kvPrev;
 			}
-			free(p);
+
+			/* move the deleted element after the end of the list */
+			p->kvPrev = list->kvEnd;
+			p->kvNext = NULL;
+			list->kvEnd->kvNext=p;
+			list->kvEnd= p;
+
 			list->count--;
 			return 0;
 		}
