@@ -1,5 +1,5 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2012.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2014.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
  also: patches by Rainer Keller, thanks a lot!
 */
@@ -43,9 +43,9 @@ int OTF_RStream_init( OTF_RStream* rstream ) {
 
 	rstream->buffersizes= 1024*1024;
 	
-#ifdef HAVE_ZLIB
-	rstream->zbuffersizes= 1024*10;
-#endif /* HAVE_ZLIB */
+/*#ifdef HAVE_ZLIB*/
+	rstream->zbuffersizes= OTF_ZBUFFER_DEFAULTSIZE;
+/*#endif*/ /* HAVE_ZLIB */
 
 	rstream->manager= NULL;
 
@@ -973,7 +973,7 @@ uint64_t OTF_RStream_readMarker( OTF_RStream* rstream, OTF_HandlerArray* handler
 		if ( NULL == rstream->markerBuffer ) {
 		
 			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
-					"the stream has no def buffer.\n",
+					"the stream has no marker buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
 			
 			/* there is no def buffer available for this stream */
