@@ -1,5 +1,5 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2012.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2014.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
 */
 
@@ -165,10 +165,10 @@ struct struct_OTF_RStream {
 	/** Default size of buffers managed by this RStream. */
 	uint32_t buffersizes;
 	
-#ifdef HAVE_ZLIB
+/*#ifdef HAVE_ZLIB*/
 	/** Default size of zbuffers managed by this RStream. */
 	uint32_t zbuffersizes;
-#endif /* HAVE_ZLIB */
+/*#endif*/ /* HAVE_ZLIB */
 	
 	/** file handle manager */
 	OTF_FileManager* manager;
