@@ -1,5 +1,5 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2012.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2014.
  Authors: Andreas Knuepfer, Robert Dietrich, Matthias Jurenz
 */
 
@@ -18,7 +18,8 @@
 
 
 using namespace std;
-
+/*store current callpath for each process  */
+map<uint32_t,string> callpathMap;
 
 static void prepare_progress( AllData& alldata, uint64_t max_bytes ) {
 
@@ -442,6 +443,17 @@ static int handle_enter( void* fha, uint64_t time, uint32_t function,
     list<StackType>& stack= alldata->stackPerProcess[ process ];
     stack.push_back( StackType( function, time ) );
 
+    if (alldata->params.dispersion.mode == DISPERSION_MODE_PERCALLPATH)
+    {
+        /* store current callpath */
+        std::ostringstream os;
+        os << " "<<function;
+        callpathMap[process] += os.str();
+        /* save maximum length, for buffer allocation in reduce_data.cpp*/
+        if(alldata->maxCallpathLength < callpathMap[process].length())
+            alldata->maxCallpathLength = callpathMap[process].length();
+    }
+    
     return OTF_RETURN_OK;
 }
 
@@ -500,12 +512,17 @@ static int handle_leave( void* fha, uint64_t time, uint32_t function,
 
     stack.pop_back();
 
-    /*
-    cerr << " func " << func << " @ process " << process << ": " << 
-        "excl " << excl << " ticks, incl " << incl << " ticks" << endl;
-    */
     alldata->functionMapPerRank[ Pair( process, func ) ].add( 1, excl, incl );
 
+    if(alldata->params.dispersion.mode == DISPERSION_MODE_PERCALLPATH)
+    {
+        /* store function by process, callpath and functionId*/
+        alldata->functionCallpathMapPerRank[ TripleCallpath( process, callpathMap[process],func ) ].add( 1, excl, incl );
+        alldata->functionCallpathMapPerRank[ TripleCallpath( process, callpathMap[process],func ) ].callpath = callpathMap[process];
+        /* reduce callpath step at leave */
+        callpathMap[process] = callpathMap[process].substr (0,callpathMap[process].find_last_of(" "));
+    }
+
     return OTF_RETURN_OK;
 }
 
@@ -602,6 +619,14 @@ static int handle_send( void* fha, uint64_t time, uint32_t sender,
 
     }
 
+    if (alldata->params.write_csv_msg_sizes)
+    {
+      alldata->messageMapPerSize[ length ].count++;
+      if ( duration > 0.0 )
+      {
+         alldata->messageMapPerSize[ length ].time.append((uint64_t)duration);
+      }
+    }
     return OTF_RETURN_OK;
 }
 
@@ -961,6 +986,11 @@ static void share_definitions( AllData& alldata ) {
 
             }
 
+            /* allProcesses */
+            MPI_Pack_size( 1 + alldata.allProcesses.size() * 2, 
+                           MPI_LONG_LONG_INT, MPI_COMM_WORLD, &size );
+            buffer_size += size;
+
         }
 
     }
@@ -1112,6 +1142,28 @@ static void share_definitions( AllData& alldata ) {
 
             }
 
+            /* allProcesses.size() */
+            uint64_t all_processes_map = alldata.allProcesses.size();
+            MPI_Pack( &all_processes_map, 1, MPI_LONG_LONG_INT, buffer, 
+                      buffer_size, &buffer_pos, MPI_COMM_WORLD );
+
+            /* allProcesses */
+            for ( set< Process, ltProcess >::iterator it = alldata.allProcesses.begin();
+                  it != alldata.allProcesses.end();
+                  it++ )
+            {
+               uint64_t process, parent;
+               process = it->process;
+               parent  = it->parent;
+
+               /* process */
+               MPI_Pack( &process, 1, MPI_LONG_LONG_INT, buffer, buffer_size, 
+                         &buffer_pos, MPI_COMM_WORLD );
+               /* parent */
+               MPI_Pack( &parent, 1, MPI_LONG_LONG_INT, buffer, buffer_size, 
+                         &buffer_pos, MPI_COMM_WORLD );
+            }
+
         }
 
     }
@@ -1265,6 +1317,28 @@ static void share_definitions( AllData& alldata ) {
 
                 delete [] second;
             }
+
+            /* allProcesses.size() */
+            uint64_t all_processes_size;
+            MPI_Unpack( buffer, buffer_size, &buffer_pos, 
+                        &all_processes_size, 1, MPI_LONG_LONG_INT,
+                        MPI_COMM_WORLD );
+
+            for ( uint64_t i = 0; i < all_processes_size; i++ )
+            {
+               uint64_t process, parent;
+
+               /* process */
+               MPI_Unpack( buffer, buffer_size, &buffer_pos, 
+                           &process, 1, MPI_LONG_LONG_INT,
+                           MPI_COMM_WORLD );
+               /* parent */
+               MPI_Unpack( buffer, buffer_size, &buffer_pos, 
+                           &parent, 1, MPI_LONG_LONG_INT,
+                           MPI_COMM_WORLD );
+
+               alldata.allProcesses.insert( Process(process, parent) );
+            }
         }
     }
 
@@ -1333,17 +1407,23 @@ static bool read_events( AllData& alldata, OTF_Reader* reader ) {
     if ( alldata.params.progress ) {
 
         OTF_Reader_setRecordLimit( reader, 0 );
+        error= ( OTF_READ_ERROR == OTF_Reader_readEvents( reader, handlers ) );
 
-        if ( OTF_READ_ERROR != OTF_Reader_readEvents( reader, handlers ) ) {
+#ifdef OTFPROFILE_MPI
+        /* synchronize error indicator to workers */
+        SyncError( alldata, error );
+#endif /* OTFPROFILE_MPI */
+
+        if ( !error ) {
 
             uint64_t min, cur, max;
 
             OTF_Reader_eventBytesProgress( reader, &min, &cur, &max );
             prepare_progress( alldata, max );
 
-        }
+            OTF_Reader_setRecordLimit( reader, Progress::EVENTS_RECORD_LIMIT );
 
-        OTF_Reader_setRecordLimit( reader, Progress::EVENTS_RECORD_LIMIT );
+        }
 
     }
 
@@ -1351,7 +1431,7 @@ static bool read_events( AllData& alldata, OTF_Reader* reader ) {
 
     uint64_t records_read= 0;
 
-    while ( OTF_READ_ERROR !=
+    while ( !error && OTF_READ_ERROR !=
             ( records_read= OTF_Reader_readEvents( reader, handlers ) ) ) {
 
         /* update progress */
@@ -1371,14 +1451,12 @@ static bool read_events( AllData& alldata, OTF_Reader* reader ) {
         if ( 0 == records_read )
             break;
     }
+    error= ( error || OTF_READ_ERROR == records_read );
 
-    /* show error message if reading failed */
-    if ( OTF_READ_ERROR == records_read ) {
-
-        cerr << "ERROR: Could not read events." << endl;
-        error= true;
-
-    }
+#ifdef OTFPROFILE_MPI
+    /* synchronize error indicator to workers */
+    SyncError( alldata, error );
+#endif /* OTFPROFILE_MPI */
 
     /* close OTF handler array */
     OTF_HandlerArray_close( handlers );
@@ -1427,24 +1505,31 @@ static bool read_statistics( AllData& alldata, OTF_Reader* reader ) {
     if ( alldata.params.progress ) {
 
         OTF_Reader_setRecordLimit( reader, 0 );
+        error=
+            ( OTF_READ_ERROR == OTF_Reader_readStatistics( reader, handlers ) );
 
-        if ( OTF_READ_ERROR != OTF_Reader_readStatistics( reader, handlers ) ) {
+#ifdef OTFPROFILE_MPI
+        /* synchronize error indicator to workers */
+        SyncError( alldata, error );
+#endif /* OTFPROFILE_MPI */
+
+        if ( !error ) {
 
             uint64_t min, cur, max;
+
             OTF_Reader_statisticBytesProgress( reader, &min, &cur, &max );
             prepare_progress( alldata, max );
 
+            OTF_Reader_setRecordLimit( reader, Progress::STATS_RECORD_LIMIT );
         }
 
-        OTF_Reader_setRecordLimit( reader, Progress::STATS_RECORD_LIMIT );
-
     }
 
     /* read statistics */
 
     uint64_t records_read= 0;
 
-    while ( OTF_READ_ERROR !=
+    while ( !error && OTF_READ_ERROR !=
             ( records_read= OTF_Reader_readStatistics( reader, handlers ) ) ) {
 
         /* update progress */
@@ -1464,14 +1549,12 @@ static bool read_statistics( AllData& alldata, OTF_Reader* reader ) {
         if ( 0 == records_read )
             break;
     }
+    error= ( error || OTF_READ_ERROR == records_read );
 
-    /* show error message if reading failed */
-    if ( OTF_READ_ERROR == records_read ) {
-
-        cerr << "ERROR: Could not read statistics." << endl;
-        error= true;
-
-    }
+#ifdef OTFPROFILE_MPI
+    /* synchronize error indicator to workers */
+    SyncError( alldata, error );
+#endif /* OTFPROFILE_MPI */
 
     /* close OTF handler array */
     OTF_HandlerArray_close( handlers );
@@ -1481,7 +1564,7 @@ static bool read_statistics( AllData& alldata, OTF_Reader* reader ) {
 
 
 bool CollectData( AllData& alldata ) {
-
+	alldata.maxCallpathLength = 0;
     bool error= false;
 
     /* start runtime measurement for collecting data */
@@ -1528,17 +1611,42 @@ bool CollectData( AllData& alldata ) {
 
         /* either read data from events or statistics */
 
-        if ( alldata.params.read_from_stats ) {
-
-            VerbosePrint( alldata, 1, true, "reading statistics\n" );
-
-            error= !read_statistics( alldata, reader );
-
-        } else {
+        if ( !alldata.params.read_from_stats ) {
 
             VerbosePrint( alldata, 1, true, "reading events\n" );
 
             error= !read_events( alldata, reader );
+            if ( error ) {
+
+                if ( 0 == alldata.myRank ) {
+
+                    cerr << "ERROR: Could not read events." << endl
+                         << "If the input trace has only summarized "
+                         << "information (statistics) and no events, "
+                         << "please try again with option '--stat'." << endl;
+
+                }
+
+                break;
+
+            }
+
+        } else {
+
+            VerbosePrint( alldata, 1, true, "reading statistics\n" );
+
+            error= !read_statistics( alldata, reader );
+            if ( error ) {
+
+                if ( 0 == alldata.myRank ) {
+
+                    cerr << "ERROR: Could not read statistics." << endl;
+
+                }
+
+                break;
+
+            }
 
         }
 
@@ -1549,11 +1657,6 @@ bool CollectData( AllData& alldata ) {
 
         }
 
-#ifdef OTFPROFILE_MPI
-        /* synchronize error indicator with workers */
-        SyncError( alldata, error );
-#endif /* OTFPROFILE_MPI */
-
     } while( false );
 
     /* close OTF file manager and reader */
