@@ -1,5 +1,5 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2013.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2014.
  Authors: Andreas Knuepfer, Robert Dietrich, Matthias Jurenz
 */
 
@@ -65,7 +65,7 @@ struct Params {
     static const bool     DEFAULT_CREATE_TEX= true;
     static const bool     DEFAULT_CREATE_PDF= true;
     static const string   DEFAULT_OUTPUT_FILE_PREFIX() { return "result"; }
-   
+
     uint32_t max_file_handles;
     uint32_t buffer_size;
     uint32_t max_groups;
@@ -77,6 +77,10 @@ struct Params {
     bool     create_csv;
     bool     create_tex;
     bool     create_pdf;
+
+    bool     write_csv_msg_matrices;
+    bool     write_csv_msg_sizes;
+
     string   input_file_prefix;
     string   output_file_prefix;
 
@@ -86,7 +90,6 @@ struct Params {
 
         static const ClusterAlgorithm DEFAULT_ALGORITHM= CLUSTER_ALG_KMEANS;
         static double DEFAULT_QUALITY_THRESHOLD() { return 0.1; }
-        static const string DEFAULT_MAP_FILE_NAME() { return "result.map"; }
 
         ClusterAlgorithm alg;
         bool             enabled;
@@ -94,7 +97,6 @@ struct Params {
         bool             hard_grouping;
         double           quality_threshold;
 
-        string           map_file_name;
         string           shrink_output_prefix;
 
         bool             synth_data;
@@ -105,11 +107,12 @@ struct Params {
            : alg(DEFAULT_ALGORITHM), enabled(false), shrink(false),
              hard_grouping(false),
              quality_threshold(DEFAULT_QUALITY_THRESHOLD()),
-             map_file_name(DEFAULT_MAP_FILE_NAME()), synth_data(false),
-             synth_ranks_num(0), synth_funcs_num(0) {}
+             synth_data(false), synth_ranks_num(0), synth_funcs_num(0) {}
 
     } clustering;
 
+    /* irregularity analysis related parameters */
+
     struct Dispersion {
 
         static const DispersionMode    DEFAULT_MODE= DISPERSION_MODE_PERFUNCTION;
@@ -135,6 +138,7 @@ struct Params {
           verbose_level(DEFAULT_VERBOSE_LEVEL), progress(false),
           read_from_stats(false), create_csv(DEFAULT_CREATE_CSV), 
           create_tex(DEFAULT_CREATE_TEX), create_pdf(DEFAULT_CREATE_PDF),
+          write_csv_msg_matrices(false), write_csv_msg_sizes(false),
           output_file_prefix(DEFAULT_OUTPUT_FILE_PREFIX()) {}
 
 };
@@ -876,6 +880,16 @@ struct MessageSpeedData {
 typedef MessageData CollectiveData;
 
 
+struct MessageSizeData {
+
+    uint64_t              count;
+    min_max_avg<uint64_t> time;
+
+    MessageSizeData() : count(0) {}
+    ~MessageSizeData() {}
+};
+
+
 struct PendingCollective {
 
     uint32_t collop;
@@ -1018,6 +1032,9 @@ struct AllData {
     communication partners */
     map< uint64_t, MessageData > messageMapPerRank;
 
+    /* store statistics for each message size */
+    map< uint64_t, MessageSizeData > messageMapPerSize;
+
     /* store per-collop.-class statistics over the ranks,
     Pair is <rank,collective-class> */
     map< Pair, CollectiveData, ltPair > collectiveMapPerRank;
@@ -1025,7 +1042,7 @@ struct AllData {
     /* store per-function dispersion information over the rank, Triple is 
     <bin,funcId,rank> */
     map< Triple, FunctionDispersionData, ltTriple > functionDispersionMapPerRank;
-    
+
 
     /* data summarization and reduction containers:
     the following maps are filled when summarizing the previous set of maps,
